from multiprocessing.pool import Pool
import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb
import tkinter as tk
import pystan
import pickle
import platform

stan_code = """
    data {
        int N;
        vector[2] xy[N];
    }
    parameters {
        real mu_x;
        real mu_y;
        real<lower = 0.0> sgm_x;
        real<lower = 0.0> sgm_y;
        real<lower = -1.0, upper = 1.0> r;
    }
    model {
        mu_x ~ normal(0.0, 1000.0);
        mu_y ~ normal(0.0, 1000.0);
        sgm_x ~ exponential(0.001);
        sgm_y ~ exponential(0.001);
        r ~ uniform(-1.0, 1.0);
        xy ~ multi_normal([mu_x, mu_y],
                          [[sgm_x*sgm_x, sgm_x*sgm_y*r],
                           [sgm_x*sgm_y*r, sgm_y*sgm_y]]);  
    }
"""

def do_mcmc(smpl_s, rho):
    root = tk.Tk()
    root.title(f'sample size: {smpl_s},  rho = {rho}')
    cnvs = tk.Canvas(width = 700, height = 300, background = '#00ffff')
    cnvs.pack()
    cnvs.create_text(350, 150, text = '', font = ('', 100), fill = 'blue',
                     tags = 'MyText')
    
    fit_list = []
    for t in range(20):
        cnvs.itemconfig('MyText', text = f'{t+1}/20')
        cnvs.update()
        
        xy = ss.multivariate_normal([0, 0],
                            [[1.0, rho], [rho, 1.0]]).rvs(size = smpl_s)
        with open('sm.pkl', 'rb') as f:
                sm = pickle.load(f)

        if platform.system() == 'Windows':
                fit = sm.sampling(data = {'N':len(xy), 'xy':xy}, n_jobs = 1)
        else:
                fit = sm.sampling(data = {'N':len(xy), 'xy':xy})

        fit_list.append(fit['r'])

    root.destroy()
    return smpl_s, rho, fit_list


params = [{'smpl_s':10, 'rho':0.0},
          {'smpl_s':10, 'rho':0.5},
          {'smpl_s':100, 'rho':0.0},
          {'smpl_s':100, 'rho':0.5}]

def funcs(params):
    rslts = do_mcmc(params['smpl_s'], params['rho'])
    return rslts


if __name__ == '__main__':
    
    sm = pystan.StanModel(model_code = stan_code)

    with open('sm.pkl', 'wb') as f:
        pickle.dump(sm, f)
 	    
    pool = Pool()
    Results = pool.map(funcs, params)
    pool.close()

    plt.figure(figsize = (8, 6))
    ip = 0
    for v in Results:
        ip += 1
        plt.subplot(2,2,ip) 
        plt.title(f'Sample_size = {v[0]},  rho = {v[1]}', fontsize = 14)
        dt = v[2]
        for p in dt:
            sb.kdeplot(p, clip = [-1.0, 1.0])
        plt.xticks([-1.1, -1.0, 0, 1.0, 1.1], ['', '-1', '0', '1', ''])

    plt.tight_layout()
    plt.show()
